//------------------------------------------------------------------------------
//  Demo Application02 for MSP430/DAC161P997 Interface Code Library
//  Channel inactivity error detection is enabled and timer ISR is used to send
//  two valid symbols every 75ms to keep link active
//
//                MSP430F5528
//             -----------------
//         /|\|              XIN|-
//          | |                 |  32kHz
//          --|RST          XOUT|-
//            |                 |
//            |       P2.0/TA1.1|--> PRI_TX
//            |       P2.1/TA1.2|--> PRI_TX_EN_N 
//            |       P1.7/TA1.0|<-- PRI_RX 
//            |                 | 
//            |                 | 
//            |                 | 
//
//   Vishy Natarajan
//   Texas Instruments Inc.
//   October 2011
//   Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
//------------------------------------------------------------------------------
// Change Log:
//------------------------------------------------------------------------------
// Version:  1.00
// Comments: Initial Release Version
//------------------------------------------------------------------------------

#include <stdint.h>
#include "TI_DAC161P997.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_swif.h"

void usc_init(void);                                                           // MSP430 System Clock Init 
void blink_LED(void);                                                          // Blink LED on MSP430 board few times
void timerA2_init(void);                                                       // Initialize timerA2 to generate 75ms interrupt
void run_timerA2(void);                                                        // Run timerA2 
void stop_timerA2(void);                                                       // Stop timerA2

//------------------------------------------------------------------------------
void main(void)
{
  uint8_t rx_ack;
  uint8_t ack_error_count = 0;
  
  WDTCTL = WDTPW+WDTHOLD;                                                      // Stop WDT
  usc_init();                                                                  // Initialize MSP430 System Clock  
  TI_DAC161P997_LED_PxOUT |= TI_DAC161P997_LED_PIN;                            // Set LED ON
  TI_DAC161P997_LED_PxDIR |= TI_DAC161P997_LED_PIN;                            // Set pin direction is output
    
  TI_DAC161P997_SWIFSetup();
  
  __bis_SR_register(GIE);                                                      // Enable global interrupts		

  // Test DAC config register write
  TI_DAC161P997_SWIFWriteReg(LCK_REG_UNLOCK, CONFIG_WRITE);                    // unlock config register access (tag = 1)
  TI_DAC161P997_SWIFWriteReg(TI_DAC161P997_CONFIG2_REG                         // enable ack & all error reporting including channel inactive error
                             + CONFIG2_REG_ACK_EN + CONFIG2_REG_LOOP
                             + CONFIG2_REG_FRAME  + CONFIG2_REG_PARITY
                             + CONFIG2_REG_CHANNEL, CONFIG_WRITE);     
  TI_DAC161P997_SWIFWriteReg(LCK_REG_LOCK, CONFIG_WRITE);                      // lock config register
  
  timerA2_init();

  // Test DAC output write
  while (1)
  {
    rx_ack = TI_DAC161P997_SWIFWriteReg(DACCODE_4mA, DACCODE_WRITE);           // set for 4mA current (tag = 0)
    if (rx_ack == DAC_ACK_FAIL)                                                // check if error
      ack_error_count++;                                                       // increment ack error count
    
    run_timerA2();                                                             // start timerA2: isr sends channel keep alive symbols
    blink_LED();                                                               // about 5 seconds delay      
    stop_timerA2();                                                            // stop timerA2 before sending DAC Frames
      
    rx_ack = TI_DAC161P997_SWIFWriteReg(DACCODE_12mA, DACCODE_WRITE);          // set for 12mA current (tag = 0)
    if (rx_ack == DAC_ACK_FAIL)                                                // check if error
      ack_error_count++;                                                       // increment ack error count

    run_timerA2();                                                             // start timerA2: isr sends channel keep alive symbols
    blink_LED();                                                               // about 5 seconds delay 
    stop_timerA2();                                                            // start timerA2: isr sends channel keep alive symbols

    if (ack_error_count > ACK_ERR_THRESHOLD)                                   // check if ack error count > threshold
    {
      TI_DAC161P997_SWIFWriteReg(LCK_REG_UNLOCK, CONFIG_WRITE);                // unlock config register access (tag = 1)
      TI_DAC161P997_SWIFWriteReg(TI_DAC161P997_CONFIG1_REG 
                                 + CONFIG1_REG_SET_ERR, CONFIG_WRITE);         // set error
      TI_DAC161P997_LED_PxOUT &= ~TI_DAC161P997_LED_PIN;                       // turn off LED
      break;                                                                   // exit loop
    }    
  }
  __bis_SR_register(LPM0_bits + GIE);                                          // Enter LPM0, enable interrupts
  __no_operation();                                                            // For debugger  
}


//------------------------------------------------------------------------------
//  void blink_LED(void)
//
//  DESCRIPTION:
//  Blink LED on MSP430 board few times 
//------------------------------------------------------------------------------
void blink_LED(void)
{
  uint8_t seconds = 6;

  while (seconds--)                                                            // slow blinking 3 times
  {
    __delay_cycles(CPU_CLK);                                                   // delay 1sec
    TI_DAC161P997_LED_PxOUT ^= TI_DAC161P997_LED_PIN;                          // toggle LED        
  }
}

//------------------------------------------------------------------------------
//  void usc_init(void)
//
//  DESCRIPTION:
//  MSP430 System Clock Init
//  Note: This function is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
void usc_init(void)
{
  UCSCTL3 = SELREF_2;                                                          // Set DCO FLL reference = REFO
  UCSCTL4 |= SELA_2;                                                           // Set ACLK = REFO
  UCSCTL0 = 0x0000;                                                            // Set lowest possible DCOx, MODx

  // Loop until XT1,XT2 & DCO stabilizes - In this case only DCO has to stabilize
  do
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);                                // Clear XT2,XT1,DCO fault flags
    SFRIFG1 &= ~OFIFG;                                                         // Clear fault flags
  }while (SFRIFG1&OFIFG);                                                      // Test oscillator fault flag
	
  __bis_SR_register(SCG0);                                                     // Disable the FLL control loop
  UCSCTL1 = DCORSEL_5;                                                         // Select DCO range 16MHz operation
  UCSCTL2 |= 249;                                                              // Set DCO Multiplier for 8MHz
                                                                               // (N + 1) * FLLRef = Fdco
                                                                               // (249 + 1) * 32768 = 8MHz
  __bic_SR_register(SCG0);                                                     // Enable the FLL control loop

  // Worst-case settling time for the DCO when the DCO range bits have been
  // changed is n x 32 x 32 x f_MCLK / f_FLL_reference. See UCS chapter in 5xx
  // UG for optimization.
  // 32 x 32 x 8 MHz / 32,768 Hz = 250000 = MCLK cycles for DCO to settle
  __delay_cycles(250000);
}
//------------------------------------------------------------------------------
//  void timerA2_init(void)
//
//  DESCRIPTION:
//  Initialize timer_A2 to generat 75ms interrupt. Two valid symbols are sent in
//  timer interrupt service routine to keep channel link acitve. Timer is only
//  initialized here. Separate functions provided to run and stop the timer. 
//  Note: This function is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
#define INTERVAL_75MS 9600
void timerA2_init(void)
{
  TI_DAC161P997_TIMER_INTERVAL_PxSEL &= ~TI_DAC161P997_TIMER_INTERVAL_PIN;     // Select GPIO functionality on the pin
  TI_DAC161P997_TIMER_INTERVAL_PxOUT &= ~TI_DAC161P997_TIMER_INTERVAL_PIN;     // set timer interval pin output low 
  TI_DAC161P997_TIMER_INTERVAL_PxDIR |= TI_DAC161P997_TIMER_INTERVAL_PIN;      // set timer interval pin direction is output
        
  TA2CCTL0 = CCIE;                                                             // TA2CCR0 interrupt enabled
  TA2CCR0 = INTERVAL_75MS;                                                     // Initialize for 75ms interval
  TA2EX0 = TAIDEX_7;                                                           // Divide clock (selected below) further by 8	
  TA2CTL = TASSEL_2 + MC_0 + TACLR + ID_3;                                     // SMCLK, stop, clear TAR, divide by 8
    
}
//------------------------------------------------------------------------------
//  void run_timerA2(void)
//
//  DESCRIPTION:
//  Timer is initialized to run. This function should be called before long periods
//  of inactivity so timer isr comes up periodically to send 2 valid symbols to keep
//  channel active
//  Note: This function is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
void run_timerA2(void)
{
  TA2CTL = TASSEL_2 + MC_1 + TACLR + ID_3;                                     // SMCLK, upmode, clear TAR, divide by 8
}
//------------------------------------------------------------------------------
//  void stop_timerA2(void)
//
//  DESCRIPTION:
//  Timer is stopped. This function should be called TI_DAC161P997_SWIFWriteRegister
//  so a valid DAC frame is sent. There is no need to send channel keep alive symbols
//  using timer isr when a valid DAC frames is sent.
//  Note: This function is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
void stop_timerA2(void)
{
  TA2CTL = TASSEL_2 + MC_0 + TACLR + ID_3;                                     // SMCLK, stop, clear TAR, divide by 8
  TA2CCTL0 &= ~CCIFG;                                                          // clear any pending interrupt  
}

//------------------------------------------------------------------------------
// Timer A2 interrupt service routine
// Note: This function is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
#pragma vector=TIMER2_A0_VECTOR
__interrupt void TIMER2_A0_ISR (void)
{
  TI_DAC161P997_SWIFSendSymbol(ZERO_SYM);                                      // send 2 zero symbols to keep channel active
  TI_DAC161P997_SWIFSendSymbol(ZERO_SYM);
  TI_DAC161P997_TIMER_INTERVAL_PxOUT ^= TI_DAC161P997_TIMER_INTERVAL_PIN;      // toggle timer interval pin every 75ms
  __bic_SR_register_on_exit(LPM0_bits);                                        // Exit active CPU    
}

//------------------------------------------------------------------------------
