//******************************************************************************
//  Description:  This file contains functions that allow the MSP430 device to
//  access the Single Wire Interface (SWIF) of the DAC161P997.   
//
//                MSP430F5528
//             -----------------
//         /|\|              XIN|-
//          | |                 |  32kHz
//          --|RST          XOUT|-
//            |                 |
//            |       P2.0/TA1.1|--> PRI_TX
//            |       P2.1/TA1.2|--> PRI_TX_EN_N 
//            |       P1.7/TA1.0|<-- PRI_RX 
//            |                 | 
//            |                 | 
//            |                 | 
//
//  MSP430/DAC161P997 Interface Code Library
//
//  Vishy Natarajan
//  Texas Instruments Inc.
//  January 2012
//  Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
//
//------------------------------------------------------------------------------
// Change Log:
//------------------------------------------------------------------------------
// Version:  1.00
// Comments: Initial Release Version
//------------------------------------------------------------------------------
/*  Copyright 2011-2012 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller which is 
  integrated into your product. Other than for the foregoing purpose, you may 
  not use, reproduce, copy, prepare derivative works of, modify, distribute, 
  perform, display or sell this Software and/or its documentation for any 
  purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
--------------------------------------------------------------------------------*/

#include <stdint.h>
#include "TI_DAC161P997.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_swif.h"

//------------------------------------------------------------------------------
//  void TI_DAC161P997_SWIFSetup(void)
//
//  DESCRIPTION:
//  Configures the assigned port pins to function as a SWIF port and 
//  initializes it.
//------------------------------------------------------------------------------
void TI_DAC161P997_SWIFSetup(void)
{
  
  TI_DAC161P997_SWIF_PRIRX_PxSEL &= ~TI_DAC161P997_SWIF_PRIRX_PIN;             // ensure GPIO function is selected for the pin
  TI_DAC161P997_SWIF_PRIRX_PxDIR &= ~TI_DAC161P997_SWIF_PRIRX_PIN;             // ensure pin is input
  
  TI_DAC161P997_SWIF_PRITXENN_PxSEL &= ~TI_DAC161P997_SWIF_PRITXENN_PIN;       // ensure GPIO function is selected for the pin
  TI_DAC161P997_SWIF_PRITXENN_PxOUT &= ~TI_DAC161P997_SWIF_PRITXENN_PIN;       // set PRI_TX_EN_N low
  TI_DAC161P997_SWIF_PRITXENN_PxDIR |= TI_DAC161P997_SWIF_PRITXENN_PIN;        // select pin as output
  
  TI_DAC161P997_SWIF_PRITX_PxSEL &= ~TI_DAC161P997_SWIF_PRITX_PIN;             // ensure GPIO function is selected for the pin
  TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;             // set PRI_TX output low
  TI_DAC161P997_SWIF_PRITX_PxDIR |= TI_DAC161P997_SWIF_PRITX_PIN;              // select pin as output

}

//------------------------------------------------------------------------------
//  int8_t TI_DAC161P997_SWIFWriteReg(uint16_t data, uint8_t tag)
//
//  DESCRIPTION:
//  Writes 16-bit value "data" to DAC CODE register if "tag" is 0. If "tag" is 1, configuration 
//  register value is updated with "data", in which case, the most significant byte of "data" 
//  indicates the configuration register and least significant byte of "data" is the register value. 
//  Data bits are sent over the SWIF as per the frame format described in DAC161P997 Data Sheet. 
//  Following a valid frame the SWIF is monitored for a receive acknowledgement symbol from DAC161P997. 
//  If SWIF transfer is not successful (slave rx ack failed) or acknowledgement bit ACK_EN is 
//  not enabled, an error code DAC_ACK_FAIL is returned.
//------------------------------------------------------------------------------
int8_t TI_DAC161P997_SWIFWriteReg(uint16_t data, uint8_t tag)
{
  uint8_t plow, phigh;
  uint16_t i, tmp;
  int8_t rxack_err = DAC_ACK_PASS;
        
  /* compute parity low */
  tmp = data & 0x00ff;                                                         // get least significant byte
  for (plow=0; tmp != 0; tmp >>= 1)                                            
    if (tmp & 0x1)                                                             // test if lsb is 1                                                          
      plow++;                                                                  // count number of bits equal to 1
  
  if (plow & 0x1)                                                              // check if number of 1s is odd        
    plow = 0;                                                                  // set even parity
  else
    plow = 1;                                                                  // else set odd parity
		
  /* compute parity high */
  tmp = (data & 0xff00)>>8;                                                    // get most significant byte
  for (phigh=0; tmp != 0; tmp >>= 1)
    if (tmp & 0x1)                                                             // test if lsb is 1 
      phigh++;                                                                 // count number of bits equal to 1 
  
  if (phigh & 0x1)                                                             // check if number of 1s is odd
    phigh = 0;                                                                 // set even parity
  else
    phigh = 1;                                                                 // set odd parity
		
  phigh = phigh ^ tag;                                                         // parity high is for high slice = tag + 1 byte
        
  TI_DAC161P997_SWIFSendSymbol(IDLE_SYM);                                      // Frame start: send an idle symbol first 
   
  TI_DAC161P997_SWIFSendSymbol(tag);                                           // send config bit

  tmp = data;
  for(i=0; i < 16; i++)                                                        // send 16 data bits msb to lsb
  {
    if (tmp & 0x8000)                                                          // test if msb is 1 or 0
      TI_DAC161P997_SWIFSendSymbol(ONE_SYM);                                   // send 1
    else
      TI_DAC161P997_SWIFSendSymbol(ZERO_SYM);                                  // send 0
    tmp = tmp << 1;                                                            // move next data bit to msb
  }

  TI_DAC161P997_SWIFSendSymbol(phigh);                                         // send parity high bit
  TI_DAC161P997_SWIFSendSymbol(plow);                                          // send parity low bit
	
  TI_DAC161P997_SWIFSendSymbol(IDLE_SYM);                                      // send idle

  rxack_err = TI_DAC161P997_SWIFReceiveACK();                                  // receive ack
  
  TI_DAC161P997_SWIFSendSymbol(STATIC_LOW_SYM);                                // send static low after receive ack
  
  return rxack_err;
	
}
//------------------------------------------------------------------------------
//  int8_t TI_DAC161P997_SWIFReceiveACK(void)
//
//  DESCRIPTION:
//  Check for acknowledge pulse from DAC161P997 on the SWIF RX pin. 
//  Returns an error code 1 if DAC161P997 fails to send an acknowledge pulse within 
//  the symbol period. Returns 0 if acknowledge pulse is received.  
//------------------------------------------------------------------------------
int8_t TI_DAC161P997_SWIFReceiveACK(void)
{
  uint8_t rack_err;
  
  TI_DAC161P997_SWIF_PRITXENN_PxOUT |= TI_DAC161P997_SWIF_PRITXENN_PIN;        // deassert PRI_TX_EN_N
  __delay_cycles(DUTY_CYCLE_100);                                              // center of receive period
  if  (TI_DAC161P997_SWIF_PRIRX_PxIN & TI_DAC161P997_SWIF_PRIRX_PIN)           // check if PRIRX_PIN is high
    rack_err = DAC_ACK_PASS;                                                   // clear error flag
  else
    rack_err = DAC_ACK_FAIL;                                                   // set error flag
  __delay_cycles(DUTY_CYCLE_100);                                              // total receive period is 2x duty cycle
  TI_DAC161P997_SWIF_PRITXENN_PxOUT &= ~TI_DAC161P997_SWIF_PRITXENN_PIN;       // assert PRI_TX_EN_N
  TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;             // clear PRI_TX
  return rack_err;
}
//------------------------------------------------------------------------------

//******************************************************************************
// Support for GPIO Bit Bang
//******************************************************************************
#if TI_DAC161P997_SER_INTF == TI_DAC161P997_SER_INTF_BITBANG

//------------------------------------------------------------------------------
//  void TI_DAC161P997_SWIFSendSymbol(uint8_t sym)
//
//  DESCRIPTION:
//  Send symbol "sym" to DAC161P997 on the SWIF TX pin. Valid symbols and symbol
//  periods are described in the DAC161P997 Data Sheet Table 1 & also TI_DAC161P997.h
//------------------------------------------------------------------------------
void TI_DAC161P997_SWIFSendSymbol(uint8_t sym)
{
  TI_DAC161P997_SWIF_PRITXENN_PxOUT &= ~TI_DAC161P997_SWIF_PRITXENN_PIN;       // assert PRI_TX_EN_N	
  switch (sym)
  {
  case ZERO_SYM:
    {
      TI_DAC161P997_SWIF_PRITX_PxOUT |= TI_DAC161P997_SWIF_PRITX_PIN;          // set PRI_TX output to 1
      __delay_cycles(DUTY_CYCLE_25);                                           // high period for symbol 0
      break;
    }
  case ONE_SYM:
    {
      TI_DAC161P997_SWIF_PRITX_PxOUT |= TI_DAC161P997_SWIF_PRITX_PIN;          // set PRI_TX output to 1
      __delay_cycles(DUTY_CYCLE_75);                                           // high period for symbol 1
      break;
    }
  case IDLE_SYM:
    {
      TI_DAC161P997_SWIF_PRITX_PxOUT |= TI_DAC161P997_SWIF_PRITX_PIN;          // set PRI_TX output to 1
      __delay_cycles(DUTY_CYCLE_50);                                           // high period for symbol idle
      break;
    }
  default:
    {
      TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;         // static low symbol: set PRI_TX output to 0
      __delay_cycles(DUTY_CYCLE_50);                                           // all low period
      break;
    }
  }
    
  switch (sym)
  {
  case ZERO_SYM:
    {
      TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;         // set PRI_TX to 0
      __delay_cycles(DUTY_CYCLE_75);                                           // low period for symbol 0
      break;
    }
  case ONE_SYM:
    {
      TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;         // set PRI_TX to 0
      __delay_cycles(DUTY_CYCLE_25);                                           // low period for symbol 1
      break;
    }
  case IDLE_SYM:
    {
      TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;         // set PRI_TX to 0
      __delay_cycles(DUTY_CYCLE_50);                                           // low period for idle symbol
      break;
    }
  default:
    {
      TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;         // static low symbol: set PRI_TX to 0
      __delay_cycles(DUTY_CYCLE_50);                                           // all low period
      break;
    }
  }
  
}
//------------------------------------------------------------------------------

//******************************************************************************
// Support for 552x Timer_A0
//******************************************************************************
#elif TI_DAC161P997_SER_INTF == TI_DAC161P997_SER_INTF_TIMER_A0_5xx

//------------------------------------------------------------------------------
//  void TI_DAC161P997_SWIFSendSymbol(uint8_t sym)
//
//  DESCRIPTION:
//  Send symbol "sym" to DAC161P997 on the SWIF TX pin. Valid symbols and symbol
//  periods are described in TI_DAC161P997.h
//------------------------------------------------------------------------------
void TI_DAC161P997_SWIFSendSymbol(uint8_t sym)
{
  switch (sym)
  {
    case ZERO_SYM: 
    { 
      TA0CCR0 = DUTY_CYCLE_25;                                                 // init to high period for symbol 0
      break;
    }
    case ONE_SYM:
    {
      TA0CCR0 = DUTY_CYCLE_75;                                                 // init to high period for symbol 1
      break;
    }
    case IDLE_SYM:
    {
      TA0CCR0 = DUTY_CYCLE_50;                                                 // init to high period for idle symbol
      break;
    }
    default:
    {
      TA0CCR0 = 0;                                                             // init to high period for static low symbol
      break;
     }
  }
  TI_DAC161P997_SWIF_PRITXENN_PxOUT &= ~TI_DAC161P997_SWIF_PRITXENN_PIN;       // ensure PRI_TX_EN_N is low
  TA0CTL = TASSEL_2 + MC_1 + TACLR;                                            // SMCLK, upmode, clear TAR 	
  if (TA0CCR0)                                                                 // check if high period is not 0
  {
    TI_DAC161P997_SWIF_PRITX_PxOUT |= TI_DAC161P997_SWIF_PRITX_PIN;            // set PRI_TX output high
    while (!(TA0CCTL0 & CCIFG));                                               // wait for high period completion
    TA0CCTL0 &= ~CCIFG;                                                        // clear Flag
    
    TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;           // set PRI_TX output low
    TA0CCR0 = DUTY_CYCLE_100 - TA0CCR0;                                        // init to symbol low period
    while (!(TA0CCTL0 & CCIFG));                                               // wait for low period completion
    TA0CCTL0 &= ~CCIFG;                                                        // clear flag
    
    TA0CTL = TASSEL_2 + MC_0 + TACLR;                                          // SMCLK, Stop, Clear TAR
   } else
      TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;         // set PRI_TX output low;
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

//******************************************************************************
// Support for 552x Timer_B0
//******************************************************************************
#elif TI_DAC161P997_SER_INTF == TI_DAC161P997_SER_INTF_TIMER_B0_5xx

//------------------------------------------------------------------------------
//  void TI_DAC161P997_SWIFSendSymbol(uint8_t sym)
//
//  DESCRIPTION:
//  Send symbol "sym" to DAC161P997 on the SWIF TX pin. Valid symbols and symbol
//  periods are described in TI_DAC161P997.h
//------------------------------------------------------------------------------
void TI_DAC161P997_SWIFSendSymbol(uint8_t sym)
{
  switch (sym)
  {
    case ZERO_SYM: 
    { 
      TB0CCR0 = DUTY_CYCLE_25;                                                 // init to high period for symbol 0
      break;
    }
    case ONE_SYM:
    {
      TB0CCR0 = DUTY_CYCLE_75;                                                 // init to high period for symbol 1
      break;
    }
    case IDLE_SYM:
    {
      TB0CCR0 = DUTY_CYCLE_50;                                                 // init to high period for idle symbol
      break;
    }
    default:
    {
      TB0CCR0 = 0;                                                             // init to high period for static low symbol
      break;
     }
  }
  TI_DAC161P997_SWIF_PRITXENN_PxOUT &= ~TI_DAC161P997_SWIF_PRITXENN_PIN;       // ensure PRI_TX_EN_N is low
  TB0CTL = TASSEL_2 + MC_1 + TACLR;                                            // SMCLK, upmode, clear TAR 	
  if (TB0CCR0)                                                                 // check if high period is not 0
  {
    TI_DAC161P997_SWIF_PRITX_PxOUT |= TI_DAC161P997_SWIF_PRITX_PIN;            // set PRI_TX output high
    while (!(TB0CCTL0 & CCIFG));                                               // wait for high period completion
    TB0CCTL0 &= ~CCIFG;                                                        // clear flag
    
    TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;           // set PRI_TX output low
    TB0CCR0 = DUTY_CYCLE_100 - TB0CCR0;                                        // init to symbol low period
    while (!(TB0CCTL0 & CCIFG));                                               // wait for low period completion
    TB0CCTL0 &= ~CCIFG;                                                        // clear flag
    
    TB0CTL = TASSEL_2 + MC_0 + TACLR;                                          // SMCLK, Stop, Clear TAR
   } else
      TI_DAC161P997_SWIF_PRITX_PxOUT &= ~TI_DAC161P997_SWIF_PRITX_PIN;         // set PRI_TX output low;
}
//------------------------------------------------------------------------------
#endif
